@@ -1,8 +1,5 @@
-pub mod chat;
 pub(crate) mod headers;
 pub mod responses;
 
-pub use chat::ChatRequest;
-pub use chat::ChatRequestBuilder;
 pub use responses::ResponsesRequest;
 pub use responses::ResponsesRequestBuilder;
