@@ -165,6 +165,12 @@ pub struct ResponsesStreamEvent {
     content_index: Option<i64>,
 }
 
+impl ResponsesStreamEvent {
+    pub fn kind(&self) -> &str {
+        &self.kind
+    }
+}
+
 #[derive(Debug)]
 pub enum ResponsesEventError {
     Api(ApiError),
