@@ -7,6 +7,7 @@ use std::fs::{self};
 use std::io::Write;
 use std::path::Path;
 
+use chrono::TimeZone;
 use pretty_assertions::assert_eq;
 use tempfile::TempDir;
 use time::Duration;
@@ -22,6 +23,7 @@ use crate::rollout::list::ThreadItem;
 use crate::rollout::list::ThreadSortKey;
 use crate::rollout::list::ThreadsPage;
 use crate::rollout::list::get_threads;
+use crate::rollout::recorder::RolloutRecorder;
 use crate::rollout::rollout_date_parts;
 use anyhow::Result;
 use codex_protocol::ThreadId;
@@ -45,6 +47,212 @@ fn provider_vec(providers: &[&str]) -> Vec<String> {
         .collect()
 }
 
+async fn insert_state_db_thread(
+    home: &Path,
+    thread_id: ThreadId,
+    rollout_path: &Path,
+    archived: bool,
+) {
+    let runtime =
+        codex_state::StateRuntime::init(home.to_path_buf(), TEST_PROVIDER.to_string(), None)
+            .await
+            .expect("state db should initialize");
+    let created_at = chrono::Utc
+        .with_ymd_and_hms(2025, 1, 3, 12, 0, 0)
+        .single()
+        .expect("valid datetime");
+    let mut builder = codex_state::ThreadMetadataBuilder::new(
+        thread_id,
+        rollout_path.to_path_buf(),
+        created_at,
+        SessionSource::Cli,
+    );
+    builder.model_provider = Some(TEST_PROVIDER.to_string());
+    builder.cwd = home.to_path_buf();
+    if archived {
+        builder.archived_at = Some(created_at);
+    }
+    let mut metadata = builder.build(TEST_PROVIDER);
+    metadata.has_user_event = true;
+    runtime
+        .upsert_thread(&metadata)
+        .await
+        .expect("state db upsert should succeed");
+}
+
+#[tokio::test]
+async fn list_threads_prefers_state_db_when_available() {
+    let temp = TempDir::new().unwrap();
+    let home = temp.path();
+    let fs_uuid = Uuid::from_u128(101);
+    write_session_file(
+        home,
+        "2025-01-03T13-00-00",
+        fs_uuid,
+        1,
+        Some(SessionSource::Cli),
+    )
+    .unwrap();
+
+    let db_uuid = Uuid::from_u128(102);
+    let db_thread_id = ThreadId::from_string(&db_uuid.to_string()).expect("valid thread id");
+    let db_rollout_path = home.join(format!(
+        "sessions/2025/01/03/rollout-2025-01-03T12-00-00-{db_uuid}.jsonl"
+    ));
+    insert_state_db_thread(home, db_thread_id, db_rollout_path.as_path(), false).await;
+
+    let page = RolloutRecorder::list_threads(
+        home,
+        10,
+        None,
+        ThreadSortKey::CreatedAt,
+        NO_SOURCE_FILTER,
+        None,
+        TEST_PROVIDER,
+    )
+    .await
+    .expect("thread listing should succeed");
+
+    assert_eq!(page.items.len(), 1);
+    assert_eq!(page.items[0].path, db_rollout_path);
+}
+
+#[tokio::test]
+async fn list_archived_threads_prefers_state_db_when_available() {
+    let temp = TempDir::new().unwrap();
+    let home = temp.path();
+    let archived_root = home.join("archived_sessions");
+    fs::create_dir_all(&archived_root).unwrap();
+    let fs_uuid = Uuid::from_u128(201);
+    let fs_path = archived_root.join(format!("rollout-2025-01-03T13-00-00-{fs_uuid}.jsonl"));
+    fs::write(&fs_path, "{\"type\":\"session_meta\",\"payload\":{}}\n").unwrap();
+
+    let db_uuid = Uuid::from_u128(202);
+    let db_thread_id = ThreadId::from_string(&db_uuid.to_string()).expect("valid thread id");
+    let db_rollout_path =
+        archived_root.join(format!("rollout-2025-01-03T12-00-00-{db_uuid}.jsonl"));
+    insert_state_db_thread(home, db_thread_id, db_rollout_path.as_path(), true).await;
+
+    let page = RolloutRecorder::list_archived_threads(
+        home,
+        10,
+        None,
+        ThreadSortKey::CreatedAt,
+        NO_SOURCE_FILTER,
+        None,
+        TEST_PROVIDER,
+    )
+    .await
+    .expect("archived thread listing should succeed");
+
+    assert_eq!(page.items.len(), 1);
+    assert_eq!(page.items[0].path, db_rollout_path);
+}
+
+#[tokio::test]
+async fn list_threads_db_excludes_archived_entries() {
+    let temp = TempDir::new().unwrap();
+    let home = temp.path();
+    let sessions_root = home.join("sessions/2025/01/03");
+    let archived_root = home.join("archived_sessions");
+    fs::create_dir_all(&sessions_root).unwrap();
+    fs::create_dir_all(&archived_root).unwrap();
+
+    let active_uuid = Uuid::from_u128(211);
+    let active_thread_id =
+        ThreadId::from_string(&active_uuid.to_string()).expect("valid active thread id");
+    let active_rollout_path =
+        sessions_root.join(format!("rollout-2025-01-03T12-00-00-{active_uuid}.jsonl"));
+    insert_state_db_thread(home, active_thread_id, active_rollout_path.as_path(), false).await;
+
+    let archived_uuid = Uuid::from_u128(212);
+    let archived_thread_id =
+        ThreadId::from_string(&archived_uuid.to_string()).expect("valid archived thread id");
+    let archived_rollout_path =
+        archived_root.join(format!("rollout-2025-01-03T11-00-00-{archived_uuid}.jsonl"));
+    insert_state_db_thread(
+        home,
+        archived_thread_id,
+        archived_rollout_path.as_path(),
+        true,
+    )
+    .await;
+
+    let page = RolloutRecorder::list_threads(
+        home,
+        10,
+        None,
+        ThreadSortKey::CreatedAt,
+        NO_SOURCE_FILTER,
+        None,
+        TEST_PROVIDER,
+    )
+    .await
+    .expect("thread listing should succeed");
+
+    assert_eq!(page.items.len(), 1);
+    assert_eq!(page.items[0].path, active_rollout_path);
+}
+
+#[tokio::test]
+async fn list_threads_falls_back_to_files_when_state_db_is_unavailable() {
+    let temp = TempDir::new().unwrap();
+    let home = temp.path();
+    let fs_uuid = Uuid::from_u128(301);
+    write_session_file(
+        home,
+        "2025-01-03T13-00-00",
+        fs_uuid,
+        1,
+        Some(SessionSource::Cli),
+    )
+    .unwrap();
+
+    let page = RolloutRecorder::list_threads(
+        home,
+        10,
+        None,
+        ThreadSortKey::CreatedAt,
+        NO_SOURCE_FILTER,
+        None,
+        TEST_PROVIDER,
+    )
+    .await
+    .expect("thread listing should succeed");
+
+    assert_eq!(page.items.len(), 1);
+    let file_name = page.items[0]
+        .path
+        .file_name()
+        .and_then(|value| value.to_str())
+        .expect("rollout file name should be utf8");
+    assert!(
+        file_name.contains(&fs_uuid.to_string()),
+        "expected file path from filesystem listing, got: {file_name}"
+    );
+}
+
+#[tokio::test]
+async fn find_thread_path_falls_back_when_db_path_is_stale() {
+    let temp = TempDir::new().unwrap();
+    let home = temp.path();
+    let uuid = Uuid::from_u128(302);
+    let thread_id = ThreadId::from_string(&uuid.to_string()).expect("valid thread id");
+    let ts = "2025-01-03T13-00-00";
+    write_session_file(home, ts, uuid, 1, Some(SessionSource::Cli)).unwrap();
+    let fs_rollout_path = home.join(format!("sessions/2025/01/03/rollout-{ts}-{uuid}.jsonl"));
+
+    let stale_db_path = home.join(format!(
+        "sessions/2099/01/01/rollout-2099-01-01T00-00-00-{uuid}.jsonl"
+    ));
+    insert_state_db_thread(home, thread_id, stale_db_path.as_path(), false).await;
+
+    let found = crate::rollout::find_thread_path_by_id_str(home, &uuid.to_string())
+        .await
+        .expect("lookup should succeed");
+    assert_eq!(found, Some(fs_rollout_path));
+}
+
 #[test]
 fn rollout_date_parts_extracts_directory_components() {
     let file_name = OsStr::new("rollout-2025-03-01T09-00-00-123.jsonl");
