@@ -6,6 +6,7 @@ use std::sync::RwLock;
 
 use codex_utils_absolute_path::AbsolutePathBuf;
 use toml::Value as TomlValue;
+use tracing::info;
 use tracing::warn;
 
 use crate::config::Config;
@@ -51,14 +52,11 @@ impl SkillsManager {
             skill_roots_from_layer_stack_with_agents(&config.config_layer_stack, &config.cwd);
         let mut outcome = load_skills_from_roots(roots);
         outcome.disabled_paths = disabled_paths_from_stack(&config.config_layer_stack);
-        match self.cache_by_cwd.write() {
-            Ok(mut cache) => {
-                cache.insert(cwd.to_path_buf(), outcome.clone());
-            }
-            Err(err) => {
-                err.into_inner().insert(cwd.to_path_buf(), outcome.clone());
-            }
-        }
+        let mut cache = match self.cache_by_cwd.write() {
+            Ok(cache) => cache,
+            Err(err) => err.into_inner(),
+        };
+        cache.insert(cwd.to_path_buf(), outcome.clone());
         outcome
     }
 
@@ -109,22 +107,22 @@ impl SkillsManager {
         let roots = skill_roots_from_layer_stack_with_agents(&config_layer_stack, cwd);
         let mut outcome = load_skills_from_roots(roots);
         outcome.disabled_paths = disabled_paths_from_stack(&config_layer_stack);
-        match self.cache_by_cwd.write() {
-            Ok(mut cache) => {
-                cache.insert(cwd.to_path_buf(), outcome.clone());
-            }
-            Err(err) => {
-                err.into_inner().insert(cwd.to_path_buf(), outcome.clone());
-            }
-        }
+        let mut cache = match self.cache_by_cwd.write() {
+            Ok(cache) => cache,
+            Err(err) => err.into_inner(),
+        };
+        cache.insert(cwd.to_path_buf(), outcome.clone());
         outcome
     }
 
     pub fn clear_cache(&self) {
-        match self.cache_by_cwd.write() {
-            Ok(mut cache) => cache.clear(),
-            Err(err) => err.into_inner().clear(),
-        }
+        let mut cache = match self.cache_by_cwd.write() {
+            Ok(cache) => cache,
+            Err(err) => err.into_inner(),
+        };
+        let cleared = cache.len();
+        cache.clear();
+        info!("skills cache cleared ({cleared} entries)");
     }
 }
 
