@@ -14,7 +14,6 @@ use codex_protocol::protocol::RolloutItem;
 use codex_protocol::protocol::SessionSource;
 use codex_state::DB_METRIC_COMPARE_ERROR;
 pub use codex_state::LogEntry;
-use codex_state::STATE_DB_FILENAME;
 use codex_state::ThreadMetadataBuilder;
 use serde_json::Value;
 use std::path::Path;
@@ -32,7 +31,7 @@ pub(crate) async fn init_if_enabled(
     config: &Config,
     otel: Option<&OtelManager>,
 ) -> Option<StateDbHandle> {
-    let state_path = config.codex_home.join(STATE_DB_FILENAME);
+    let state_path = codex_state::state_db_path(config.codex_home.as_path());
     if !config.features.enabled(Feature::Sqlite) {
         return None;
     }
@@ -74,7 +73,7 @@ pub(crate) async fn init_if_enabled(
 
 /// Get the DB if the feature is enabled and the DB exists.
 pub async fn get_state_db(config: &Config, otel: Option<&OtelManager>) -> Option<StateDbHandle> {
-    let state_path = config.codex_home.join(STATE_DB_FILENAME);
+    let state_path = codex_state::state_db_path(config.codex_home.as_path());
     if !config.features.enabled(Feature::Sqlite)
         || !tokio::fs::try_exists(&state_path).await.unwrap_or(false)
     {
@@ -93,7 +92,7 @@ pub async fn get_state_db(config: &Config, otel: Option<&OtelManager>) -> Option
 ///
 /// This is used for parity checks during the SQLite migration phase.
 pub async fn open_if_present(codex_home: &Path, default_provider: &str) -> Option<StateDbHandle> {
-    let db_path = codex_home.join(STATE_DB_FILENAME);
+    let db_path = codex_state::state_db_path(codex_home);
     if !tokio::fs::try_exists(&db_path).await.unwrap_or(false) {
         return None;
     }
@@ -181,6 +180,59 @@ pub async fn list_thread_ids_db(
     }
 }
 
+/// List thread metadata from SQLite without rollout directory traversal.
+#[allow(clippy::too_many_arguments)]
+pub async fn list_threads_db(
+    context: Option<&codex_state::StateRuntime>,
+    codex_home: &Path,
+    page_size: usize,
+    cursor: Option<&Cursor>,
+    sort_key: ThreadSortKey,
+    allowed_sources: &[SessionSource],
+    model_providers: Option<&[String]>,
+    archived: bool,
+) -> Option<codex_state::ThreadsPage> {
+    let ctx = context?;
+    if ctx.codex_home() != codex_home {
+        warn!(
+            "state db codex_home mismatch: expected {}, got {}",
+            ctx.codex_home().display(),
+            codex_home.display()
+        );
+    }
+
+    let anchor = cursor_to_anchor(cursor);
+    let allowed_sources: Vec<String> = allowed_sources
+        .iter()
+        .map(|value| match serde_json::to_value(value) {
+            Ok(Value::String(s)) => s,
+            Ok(other) => other.to_string(),
+            Err(_) => String::new(),
+        })
+        .collect();
+    let model_providers = model_providers.map(<[String]>::to_vec);
+    match ctx
+        .list_threads(
+            page_size,
+            anchor.as_ref(),
+            match sort_key {
+                ThreadSortKey::CreatedAt => codex_state::SortKey::CreatedAt,
+                ThreadSortKey::UpdatedAt => codex_state::SortKey::UpdatedAt,
+            },
+            allowed_sources.as_slice(),
+            model_providers.as_deref(),
+            archived,
+        )
+        .await
+    {
+        Ok(page) => Some(page),
+        Err(err) => {
+            warn!("state db list_threads failed: {err}");
+            None
+        }
+    }
+}
+
 /// Look up the rollout path for a thread id using SQLite.
 pub async fn find_rollout_path_by_id(
     context: Option<&codex_state::StateRuntime>,
@@ -228,6 +280,60 @@ pub async fn persist_dynamic_tools(
     }
 }
 
+/// Get memory summaries for a thread id using SQLite.
+pub async fn get_thread_memory(
+    context: Option<&codex_state::StateRuntime>,
+    thread_id: ThreadId,
+    stage: &str,
+) -> Option<codex_state::ThreadMemory> {
+    let ctx = context?;
+    match ctx.get_thread_memory(thread_id).await {
+        Ok(memory) => memory,
+        Err(err) => {
+            warn!("state db get_thread_memory failed during {stage}: {err}");
+            None
+        }
+    }
+}
+
+/// Upsert memory summaries for a thread id using SQLite.
+pub async fn upsert_thread_memory(
+    context: Option<&codex_state::StateRuntime>,
+    thread_id: ThreadId,
+    trace_summary: &str,
+    memory_summary: &str,
+    stage: &str,
+) -> Option<codex_state::ThreadMemory> {
+    let ctx = context?;
+    match ctx
+        .upsert_thread_memory(thread_id, trace_summary, memory_summary)
+        .await
+    {
+        Ok(memory) => Some(memory),
+        Err(err) => {
+            warn!("state db upsert_thread_memory failed during {stage}: {err}");
+            None
+        }
+    }
+}
+
+/// Get the last N memories corresponding to a cwd using an exact path match.
+pub async fn get_last_n_thread_memories_for_cwd(
+    context: Option<&codex_state::StateRuntime>,
+    cwd: &Path,
+    n: usize,
+    stage: &str,
+) -> Option<Vec<codex_state::ThreadMemory>> {
+    let ctx = context?;
+    match ctx.get_last_n_thread_memories_for_cwd(cwd, n).await {
+        Ok(memories) => Some(memories),
+        Err(err) => {
+            warn!("state db get_last_n_thread_memories_for_cwd failed during {stage}: {err}");
+            None
+        }
+    }
+}
+
 /// Reconcile rollout items into SQLite, falling back to scanning the rollout file.
 pub async fn reconcile_rollout(
     context: Option<&codex_state::StateRuntime>,
