@@ -25,10 +25,7 @@ impl SessionTask for CompactTask {
         _cancellation_token: CancellationToken,
     ) -> Option<String> {
         let session = session.clone_session();
-        if crate::compact::should_use_remote_compact_task(
-            session.as_ref(),
-            &ctx.client.get_provider(),
-        ) {
+        if crate::compact::should_use_remote_compact_task(session.as_ref(), &ctx.provider) {
             let _ = session.services.otel_manager.counter(
                 "codex.task.compact",
                 1,
