@@ -11,6 +11,7 @@ use crate::tools::registry::ToolRegistry;
 use crate::tools::spec::ToolsConfig;
 use crate::tools::spec::build_specs;
 use codex_protocol::dynamic_tools::DynamicToolSpec;
+use codex_protocol::models::FunctionCallOutputBody;
 use codex_protocol::models::LocalShellAction;
 use codex_protocol::models::ResponseInputItem;
 use codex_protocol::models::ResponseItem;
@@ -181,9 +182,8 @@ impl ToolRouter {
             ResponseInputItem::FunctionCallOutput {
                 call_id,
                 output: codex_protocol::models::FunctionCallOutputPayload {
-                    content: message,
+                    body: FunctionCallOutputBody::Text(message),
                     success: Some(false),
-                    ..Default::default()
                 },
             }
         }
