@@ -81,8 +81,10 @@ mod json_result;
 mod list_dir;
 mod list_models;
 mod live_cli;
+mod live_reload;
 mod model_info_overrides;
 mod model_overrides;
+mod model_switching;
 mod model_tools;
 mod models_cache_ttl;
 mod models_etag_responses;
