@@ -8,3 +8,7 @@ pub(crate) const WEBSOCKET_REQUEST_COUNT_METRIC: &str = "codex.websocket.request
 pub(crate) const WEBSOCKET_REQUEST_DURATION_METRIC: &str = "codex.websocket.request.duration_ms";
 pub(crate) const WEBSOCKET_EVENT_COUNT_METRIC: &str = "codex.websocket.event";
 pub(crate) const WEBSOCKET_EVENT_DURATION_METRIC: &str = "codex.websocket.event.duration_ms";
+pub(crate) const RESPONSES_API_OVERHEAD_DURATION_METRIC: &str =
+    "codex.responses_api_overhead.duration_ms";
+pub(crate) const RESPONSES_API_INFERENCE_TIME_DURATION_METRIC: &str =
+    "codex.responses_api_inference_time.duration_ms";
