@@ -1,5 +1,7 @@
 mod manager_metrics;
 mod otlp_http_loopback;
+mod runtime_summary;
 mod send;
+mod snapshot;
 mod timing;
 mod validation;
