@@ -335,6 +335,7 @@ impl ModelInstructionsVariables {
     pub fn get_personality_message(&self, personality: Option<Personality>) -> Option<String> {
         if let Some(personality) = personality {
             match personality {
+                Personality::None => Some(String::new()),
                 Personality::Friendly => self.personality_friendly.clone(),
                 Personality::Pragmatic => self.personality_pragmatic.clone(),
             }
@@ -546,6 +547,10 @@ mod tests {
             model.get_model_instructions(Some(Personality::Pragmatic)),
             "Hello\n"
         );
+        assert_eq!(
+            model.get_model_instructions(Some(Personality::None)),
+            "Hello\n"
+        );
         assert_eq!(model.get_model_instructions(None), "Hello\n");
 
         let model_no_personality = test_model(Some(ModelMessages {
@@ -564,6 +569,10 @@ mod tests {
             model_no_personality.get_model_instructions(Some(Personality::Pragmatic)),
             "Hello\n"
         );
+        assert_eq!(
+            model_no_personality.get_model_instructions(Some(Personality::None)),
+            "Hello\n"
+        );
         assert_eq!(model_no_personality.get_model_instructions(None), "Hello\n");
     }
 
@@ -603,6 +612,10 @@ mod tests {
             personality_variables.get_personality_message(Some(Personality::Pragmatic)),
             Some("pragmatic".to_string())
         );
+        assert_eq!(
+            personality_variables.get_personality_message(Some(Personality::None)),
+            Some(String::new())
+        );
         assert_eq!(
             personality_variables.get_personality_message(None),
             Some("default".to_string())
@@ -621,6 +634,10 @@ mod tests {
             personality_variables.get_personality_message(Some(Personality::Pragmatic)),
             None
         );
+        assert_eq!(
+            personality_variables.get_personality_message(Some(Personality::None)),
+            Some(String::new())
+        );
         assert_eq!(
             personality_variables.get_personality_message(None),
             Some("default".to_string())
@@ -639,6 +656,10 @@ mod tests {
             personality_variables.get_personality_message(Some(Personality::Pragmatic)),
             Some("pragmatic".to_string())
         );
+        assert_eq!(
+            personality_variables.get_personality_message(Some(Personality::None)),
+            Some(String::new())
+        );
         assert_eq!(personality_variables.get_personality_message(None), None);
     }
 }
