@@ -1,9 +1,11 @@
 mod log;
+mod thread_memory;
 mod thread_metadata;
 
 pub use log::LogEntry;
 pub use log::LogQuery;
 pub use log::LogRow;
+pub use thread_memory::ThreadMemory;
 pub use thread_metadata::Anchor;
 pub use thread_metadata::BackfillStats;
 pub use thread_metadata::ExtractionOutcome;
@@ -12,6 +14,7 @@ pub use thread_metadata::ThreadMetadata;
 pub use thread_metadata::ThreadMetadataBuilder;
 pub use thread_metadata::ThreadsPage;
 
+pub(crate) use thread_memory::ThreadMemoryRow;
 pub(crate) use thread_metadata::ThreadRow;
 pub(crate) use thread_metadata::anchor_from_item;
 pub(crate) use thread_metadata::datetime_to_epoch_seconds;
